@@ -48,6 +48,8 @@ struct _IndicatorObjectPrivate {
 	/* For get_entries_default */
 	IndicatorObjectEntry entry;
 	gboolean gotten_entries;
+
+	GStrv environments;
 };
 
 #define INDICATOR_OBJECT_GET_PRIVATE(o) (INDICATOR_OBJECT(o)->priv)
@@ -58,7 +60,10 @@ enum {
 	ENTRY_REMOVED,
 	ENTRY_MOVED,
 	SCROLL,
+	SCROLL_ENTRY,
 	MENU_SHOW,
+	SHOW_NOW_CHANGED,
+	ACCESSIBLE_DESC_UPDATE,
 	LAST_SIGNAL
 };
 
@@ -89,6 +94,7 @@ indicator_object_class_init (IndicatorObjectClass *klass)
 	klass->get_label =  NULL;
 	klass->get_menu =   NULL;
 	klass->get_image =  NULL;
+	klass->get_accessible_desc = NULL;
 
 	klass->get_entries = get_entries_default;
 	klass->get_location = NULL;
@@ -145,7 +151,6 @@ indicator_object_class_init (IndicatorObjectClass *klass)
 	                                     _indicator_object_marshal_VOID__POINTER_UINT_UINT,
 	                                     G_TYPE_NONE, 3, G_TYPE_POINTER, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_NONE);
 
-
 	/**
 		IndicatorObject::scroll:
 		@arg0: The #IndicatorObject object
@@ -161,7 +166,28 @@ indicator_object_class_init (IndicatorObjectClass *klass)
 	                                G_STRUCT_OFFSET (IndicatorObjectClass, scroll),
 	                                NULL, NULL,
 	                                _indicator_object_marshal_VOID__UINT_ENUM,
-	                                G_TYPE_NONE, 2, G_TYPE_UINT, INDICATOR_OBJECT_TYPE_SCROLL_DIRECTION);
+	                                G_TYPE_NONE, 2, G_TYPE_UINT,
+	                                INDICATOR_OBJECT_TYPE_SCROLL_DIRECTION);
+
+/**
+		IndicatorObject::scroll-entry:
+		@arg0: The #IndicatorObject object
+		@arg1: A pointer to the #IndicatorObjectEntry that
+			receives the scroll event.
+		@arg2: The delta of the scroll event
+		@arg3: The orientation of the scroll event.
+
+		When the indicator receives a mouse scroll wheel event
+		from the user, this signal is emitted.
+	*/
+	signals[SCROLL_ENTRY] = g_signal_new (INDICATOR_OBJECT_SIGNAL_SCROLL_ENTRY,
+	                                G_TYPE_FROM_CLASS(klass),
+	                                G_SIGNAL_RUN_LAST,
+	                                G_STRUCT_OFFSET (IndicatorObjectClass, scroll_entry),
+	                                NULL, NULL,
+	                                _indicator_object_marshal_VOID__POINTER_UINT_ENUM,
+	                                G_TYPE_NONE, 3, G_TYPE_POINTER, G_TYPE_UINT,
+	                                INDICATOR_OBJECT_TYPE_SCROLL_DIRECTION);
 
 	/**
 		IndicatorObject::menu-show:
@@ -181,6 +207,42 @@ indicator_object_class_init (IndicatorObjectClass *klass)
 	                                   _indicator_object_marshal_VOID__POINTER_UINT,
 	                                   G_TYPE_NONE, 2, G_TYPE_POINTER, G_TYPE_UINT);
 
+	/**
+		IndicatorObject::show-now-changed:
+		@arg0: The #IndicatorObject object
+		@arg1: A pointer to the #IndicatorObjectEntry that
+			is changing it's state
+		@arg2: The state of whether the entry should be shown
+
+		Whether the entry should be shown or not has changed so we need
+		to tell whoever is displaying it.
+	*/
+	signals[SHOW_NOW_CHANGED] = g_signal_new (INDICATOR_OBJECT_SIGNAL_SHOW_NOW_CHANGED,
+	                                          G_TYPE_FROM_CLASS(klass),
+	                                          G_SIGNAL_RUN_LAST,
+	                                          G_STRUCT_OFFSET (IndicatorObjectClass, show_now_changed),
+	                                          NULL, NULL,
+	                                          _indicator_object_marshal_VOID__POINTER_BOOLEAN,
+	                                          G_TYPE_NONE, 2, G_TYPE_POINTER, G_TYPE_BOOLEAN);
+
+	/**
+		IndicatorObject::accessible-desc-update::
+		@arg0: The #IndicatorObject object
+		@arg1: A pointer to the #IndicatorObjectEntry whos
+			accessible description has been updated.
+
+		Signaled when an indicator's accessible description
+		has been updated, so that the displayer of the
+		indicator can fetch the new description.
+	*/
+	signals[ACCESSIBLE_DESC_UPDATE] = g_signal_new (INDICATOR_OBJECT_SIGNAL_ACCESSIBLE_DESC_UPDATE,
+	                                     G_TYPE_FROM_CLASS(klass),
+	                                     G_SIGNAL_RUN_LAST,
+	                                     G_STRUCT_OFFSET (IndicatorObjectClass, accessible_desc_update),
+	                                     NULL, NULL,
+	                                     g_cclosure_marshal_VOID__POINTER,
+	                                     G_TYPE_NONE, 1, G_TYPE_POINTER, G_TYPE_NONE);
+
 
 	return;
 }
@@ -196,9 +258,12 @@ indicator_object_init (IndicatorObject *self)
 	self->priv->entry.menu = NULL;
 	self->priv->entry.label = NULL;
 	self->priv->entry.image = NULL;
+	self->priv->entry.accessible_desc = NULL;
 
 	self->priv->gotten_entries = FALSE;
 
+	self->priv->environments = NULL;
+
 	return;
 }
 
@@ -229,6 +294,11 @@ indicator_object_finalize (GObject *object)
 {
 	IndicatorObjectPrivate * priv = INDICATOR_OBJECT_GET_PRIVATE(object);
 
+	if (priv->environments != NULL) {
+		g_strfreev(priv->environments);
+		priv->environments = NULL;
+	}
+
 	if (priv->module != NULL) {
 		/* Wow, this is convoluted.  So basically we want to unref
 		   the module which will cause the code it included to be
@@ -376,6 +446,14 @@ get_entries_default (IndicatorObject * io)
 			return NULL;
 		}
 
+		if (class->get_accessible_desc) {
+			priv->entry.accessible_desc = class->get_accessible_desc(io);
+		}
+
+		if (priv->entry.accessible_desc == NULL) {
+			g_warning("IndicatorObject class does not have an accessible description.");
+		}
+
 		priv->gotten_entries = TRUE;
 	}
 
@@ -433,6 +511,30 @@ indicator_object_get_location (IndicatorObject * io, IndicatorObjectEntry * entr
 	return 0;
 }
 
+/**
+	indicator_object_get_show_now:
+	@io: #IndicatorObject to query
+	@entry: The #IndicatorObjectEntry to look for.
+
+	This function returns whether the entry should be shown with
+	priority on the panel.  If the object does not support checking
+	it assumes that its entries should never have priority.
+
+	Return value: Whether the entry should be shown with priority.
+*/
+guint
+indicator_object_get_show_now (IndicatorObject * io, IndicatorObjectEntry * entry)
+{
+	g_return_val_if_fail(INDICATOR_IS_OBJECT(io), 0);
+	IndicatorObjectClass * class = INDICATOR_OBJECT_GET_CLASS(io);
+
+	if (class->get_show_now) {
+		return class->get_show_now(io, entry);
+	}
+
+	return FALSE;
+}
+
 /**
 	indicator_object_entry_activate:
 	@io: #IndicatorObject to query
@@ -457,3 +559,95 @@ indicator_object_entry_activate (IndicatorObject * io, IndicatorObjectEntry * en
 	return;
 }
 
+/**
+	indicator_object_entry_close:
+	@io: #IndicatorObject to query
+	@entry: The #IndicatorObjectEntry whose menu was closed
+	@timestamp: The X11 timestamp of the event
+
+	Used to tell the indicator that a menu has been closed for the
+	entry that is specified.
+*/
+void
+indicator_object_entry_close (IndicatorObject * io, IndicatorObjectEntry * entry, guint timestamp)
+{
+	g_return_if_fail(INDICATOR_IS_OBJECT(io));
+	IndicatorObjectClass * class = INDICATOR_OBJECT_GET_CLASS(io);
+
+	if (class->entry_close != NULL) {
+		return class->entry_close(io, entry, timestamp);
+	}
+
+	return;
+}
+
+/**
+	indicator_object_set_environment:
+	@io: #IndicatorObject to set on
+	@env: List of enviroment names to use
+
+	Sets the names of the environment that the indicator is being
+	loaded into.  This allows for indicators to behave differently
+	in different hosts if need be.
+*/
+void
+indicator_object_set_environment (IndicatorObject * io, const GStrv env)
+{
+	g_return_if_fail(INDICATOR_IS_OBJECT(io));
+
+	if (io->priv->environments != NULL) {
+		g_strfreev(io->priv->environments);
+		io->priv->environments = NULL;
+	}
+
+	io->priv->environments = g_strdupv(env);
+
+	return;
+}
+
+/**
+	indicator_object_get_environment:
+	@io: #IndicatorObject to get the environment from
+
+	Gets the list of environment strings that this object is
+	placed into.
+
+	Return value: (transfer none): Gets the list of strings that
+	represent the environment or NULL if none were given.
+*/
+const GStrv
+indicator_object_get_environment (IndicatorObject * io)
+{
+	g_return_val_if_fail(INDICATOR_IS_OBJECT(io), NULL);
+	return io->priv->environments;
+}
+
+/**
+	indicator_object_check_environment:
+	@io: #IndicatorObject to check on
+	@env: Environment that we're looking for
+
+	Convience function to check to see if the specified environment
+	@env is in our list of environments.
+
+	Return Value: Whether we're in environment @env
+*/
+gboolean
+indicator_object_check_environment (IndicatorObject * io, const gchar * env)
+{
+	g_return_val_if_fail(INDICATOR_IS_OBJECT(io), FALSE);
+	g_return_val_if_fail(env != NULL, FALSE);
+
+	if (io->priv->environments == NULL) {
+		return FALSE;
+	}
+
+	int i;
+	for (i = 0; io->priv->environments[i] != NULL; i++) {
+		if (g_strcmp0(env, io->priv->environments[i]) == 0) {
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
