@@ -51,8 +51,14 @@ typedef enum
 #define INDICATOR_OBJECT_SIGNAL_ENTRY_MOVED_ID    (g_signal_lookup(INDICATOR_OBJECT_SIGNAL_ENTRY_MOVED, INDICATOR_OBJECT_TYPE))
 #define INDICATOR_OBJECT_SIGNAL_SCROLL            "scroll"
 #define INDICATOR_OBJECT_SIGNAL_SCROLL_ID         (g_signal_lookup(INDICATOR_OBJECT_SIGNAL_SCROLL, INDICATOR_OBJECT_TYPE))
+#define INDICATOR_OBJECT_SIGNAL_SCROLL_ENTRY      "scroll-entry"
+#define INDICATOR_OBJECT_SIGNAL_SCROLL_ENTRY_ID   (g_signal_lookup(#define INDICATOR_OBJECT_SIGNAL_SCROLL_ENTRY, INDICATOR_OBJECT_TYPE))
 #define INDICATOR_OBJECT_SIGNAL_MENU_SHOW         "menu-show"
 #define INDICATOR_OBJECT_SIGNAL_MENU_SHOW_ID      (g_signal_lookup(INDICATOR_OBJECT_SIGNAL_MENU_SHOW, INDICATOR_OBJECT_TYPE))
+#define INDICATOR_OBJECT_SIGNAL_SHOW_NOW_CHANGED  "show-now-changed"
+#define INDICATOR_OBJECT_SIGNAL_SHOW_NOW_CHANGED_ID (g_signal_lookup(INDICATOR_OBJECT_SIGNAL_SHOW_NOW_CHANGED, INDICATOR_OBJECT_TYPE))
+#define INDICATOR_OBJECT_SIGNAL_ACCESSIBLE_DESC_UPDATE "accessible-desc-update"
+#define INDICATOR_OBJECT_SIGNAL_ACCESSIBLE_DESC_UPDATE_ID (g_signal_lookup(INDICATOR_OBJECT_SIGNAL_ACCESSIBLE_DESC_UPDATE, INDICATOR_OBJECT_TYPE))
 
 typedef struct _IndicatorObject        IndicatorObject;
 typedef struct _IndicatorObjectClass   IndicatorObjectClass;
@@ -71,6 +77,8 @@ typedef struct _IndicatorObjectEntry   IndicatorObjectEntry;
 	@get_menu: Gets the image for this object.  Should be set
 		to #NULL if @get_entries is set.  Should NOT ref the
 		object.
+	@get_accessible_desc: Gets the accessible descriptionfor this
+		object.
 	@get_entries: Gets all of the entires for this object returning
 		a #GList of #IndicatorObjectEntries.  The list should be
 		under the ownership of the caller but the entires will
@@ -78,10 +86,18 @@ typedef struct _IndicatorObjectEntry   IndicatorObjectEntry;
 	@get_location: Returns the location that a particular entry
 		should be placed in.  This is really only relevant for
 		indicators that have more than one entry.
+	@get_show_now: Returns whether the entry is requesting to
+		be shown "right now" in that it has something important
+		to tell the user.
+	@entry_activate: Should be called when the menus for a given
+		entry are shown to the user.
+	@entry_close: Called when the menu is closed.
 	@entry_added: Slot for #IndicatorObject::entry-added
 	@entry_removed: Slot for #IndicatorObject::entry-removed
 	@entry_moved: Slot for #IndicatorObject::entry-moved
 	@menu_show: Slot for #IndicatorObject::menu-show
+	@show_now_changed: Slot for #IndicatorObject::show-now-changed
+	@accessible_desc_update: Slot for #IndicatorObject::accessible-desc-update
 */
 struct _IndicatorObjectClass {
 	GObjectClass parent_class;
@@ -90,11 +106,14 @@ struct _IndicatorObjectClass {
 	GtkLabel * (*get_label) (IndicatorObject * io);
 	GtkImage * (*get_image) (IndicatorObject * io);
 	GtkMenu  * (*get_menu)  (IndicatorObject * io);
+	const gchar * (*get_accessible_desc) (IndicatorObject * io);
 
 	GList *    (*get_entries) (IndicatorObject * io);
 	guint      (*get_location) (IndicatorObject * io, IndicatorObjectEntry * entry);
+	gboolean   (*get_show_now) (IndicatorObject * io, IndicatorObjectEntry * entry);
 
 	void       (*entry_activate) (IndicatorObject * io, IndicatorObjectEntry * entry, guint timestamp);
+	void       (*entry_close) (IndicatorObject * io, IndicatorObjectEntry * entry, guint timestamp);
 
 	/* Signals */
 	void       (*entry_added)   (IndicatorObject * io, IndicatorObjectEntry * entry, gpointer user_data);
@@ -102,6 +121,9 @@ struct _IndicatorObjectClass {
 	void       (*entry_moved)   (IndicatorObject * io, IndicatorObjectEntry * entry, guint old_pos, guint new_pos, gpointer user_data);
 	void       (*scroll)        (IndicatorObject * io, gint delta, IndicatorScrollDirection direction);
 	void       (*menu_show)     (IndicatorObject * io, IndicatorObjectEntry * entry, guint timestamp, gpointer user_data);
+	void       (*show_now_changed) (IndicatorObject * io, IndicatorObjectEntry * entry, gboolean show_now_state, gpointer user_data);
+	void       (*scroll_entry)  (IndicatorObject * io, IndicatorObjectEntry * entry, gint delta, IndicatorScrollDirection direction);
+	void       (*accessible_desc_update) (IndicatorObject * io, IndicatorObjectEntry * entry, gpointer user_data);
 
 	/* Reserved */
 	void       (*reserved1)     (void);
@@ -128,11 +150,24 @@ struct _IndicatorObject {
 	@label: The label to be shown on the panel
 	@image: The image to be shown on the panel
 	@menu: The menu to be added to the menubar
+	@accessible_desc: The accessible description
+		of the indicator
+
+	@reserved1: Reserved for future use
+	@reserved2: Reserved for future use
+	@reserved3: Reserved for future use
+	@reserved4: Reserved for future use
 */
 struct _IndicatorObjectEntry {
 	GtkLabel * label;
 	GtkImage * image;
 	GtkMenu  * menu;
+	const gchar * accessible_desc;
+
+	void       (*reserved1)     (void);
+	void       (*reserved2)     (void);
+	void       (*reserved3)     (void);
+	void       (*reserved4)     (void);
 };
 
 GType indicator_object_get_type (void);
@@ -140,7 +175,13 @@ IndicatorObject * indicator_object_new_from_file (const gchar * file);
 
 GList * indicator_object_get_entries (IndicatorObject * io);
 guint   indicator_object_get_location (IndicatorObject * io, IndicatorObjectEntry * entry);
+guint   indicator_object_get_show_now (IndicatorObject * io, IndicatorObjectEntry * entry);
 void    indicator_object_entry_activate (IndicatorObject * io, IndicatorObjectEntry * entry, guint timestamp);
+void    indicator_object_entry_close (IndicatorObject * io, IndicatorObjectEntry * entry, guint timestamp);
+
+void    indicator_object_set_environment (IndicatorObject * io, const GStrv env);
+const GStrv indicator_object_get_environment (IndicatorObject * io);
+gboolean indicator_object_check_environment (IndicatorObject * io, const gchar * env);
 
 G_END_DECLS
 
